// -*-C++-*-
// coord.cpp (imported from forge/servers/pangea/MapLaw/MapCoord.cc)
//
//  The WorldForge Project
//  Copyright (C) 2000  The WorldForge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about WorldForge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//

#include <math.h>  // For fabs ()
#include "coord.h"

CoalCoord::CoalCoord()
{
    for (int i = 0; i < COAL_AXIS_COUNT; i++)
        m_point[i] = 0;

    m_valid = true;  // ...until proven guilty.
}

CoalCoord::~CoalCoord()
{
}

// What do we do with m_valid??  Recalculate it, but we
//  can only do that within the CoalCoordSystem, so we'll
//  have to leave it alone for now
  
const CoalCoord CoalCoord::operator+(const CoalCoord &coord2)
{
    CoalCoord newCoord;

    for (int axis = 0; axis < COAL_AXIS_COUNT; axis++)
        newCoord.SetPoint(axis, GetPoint(axis) + coord2.GetPoint(axis));

    return newCoord;
}

const CoalCoord CoalCoord::operator-(const CoalCoord &coord2)
{
    CoalCoord newCoord;

    for (int axis = 0; axis < COAL_AXIS_COUNT; axis++)
	newCoord.SetPoint(axis, GetPoint(axis) - coord2.GetPoint(axis));

    return newCoord;
}

CoalCoord& CoalCoord::operator=(const CoalCoord& copy)
{
    // Compare pointer addresses
    if (this == &copy)
	return *this;

    for (int axis = 0; axis < COAL_AXIS_COUNT; axis++)
	SetPoint(axis, copy.GetPoint(axis));

    m_valid = copy.IsValid();
    return *this;
}

CoalCoord& CoalCoord::operator+=(const CoalCoord &coord2)
{
    for (int axis = 0; axis < COAL_AXIS_COUNT; axis++)
	SetPoint(axis, GetPoint(axis) + coord2.GetPoint(axis));

    return *this;
}

CoalCoord& CoalCoord::operator-=(const CoalCoord &coord2)
{
    for (int axis = 0; axis < COAL_AXIS_COUNT; axis++)
	SetPoint(axis, GetPoint(axis) - coord2.GetPoint(axis));

    return *this;
}

bool CoalCoord::operator==(CoalCoord& coord2)
{
    return (GetX() == coord2.GetX() && GetY() == coord2.GetY() &&
	    GetZ() == coord2.GetZ());
}

bool CoalCoord::operator==(const CoalCoord& coord2) const
{
    return (GetX() == coord2.GetX() && GetY() == coord2.GetY() &&
	    GetZ() == coord2.GetZ());
}

bool CoalCoord::operator!=(CoalCoord& coord2)
{
    return (GetX() != coord2.GetX() || GetY() != coord2.GetY() ||
	    GetZ() != coord2.GetZ());
}

bool CoalCoord::operator!=(const CoalCoord& coord2) const
{
    return (GetX() != coord2.GetX() || GetY() != coord2.GetY() ||
	    GetZ() != coord2.GetZ());
}

bool CoalCoord::operator> (CoalCoord& coord2)
{
    return (GetX() > coord2.GetX() && GetY() > coord2.GetY() &&
	    GetZ() > coord2.GetZ());
}

bool CoalCoord::operator>= (CoalCoord& coord2)
{
    return (GetX() >= coord2.GetX() && GetY() >= coord2.GetY() &&
	    GetZ() >= coord2.GetZ());
}

bool CoalCoord::operator< (CoalCoord& coord2)
{
    return (GetX() < coord2.GetX() && GetY() < coord2.GetY() &&
	    GetZ() < coord2.GetZ());
}

bool CoalCoord::operator<= (CoalCoord& coord2)
{
    return (GetX() <= coord2.GetX() && GetY() <= coord2.GetY() &&
	    GetZ() <= coord2.GetZ());
}

void CoalCoord::Add(CoalCoord coord)
{
    for (int i = 0; i < COAL_AXIS_COUNT; i++)
	SetPoint(i, GetPoint(i) + coord.GetPoint(i));
}

void CoalCoord::Subtract(CoalCoord coord)
{
    for (int i = 0; i < COAL_AXIS_COUNT; i++)
	SetPoint(i, GetPoint(i) - coord.GetPoint(i));
}

CoalCoordType CoalCoord::GetPoint (int axis) const
{
    if (axis < 0 || axis >= COAL_AXIS_COUNT)
	return 0;

    return m_point[axis];
}

void CoalCoord::SetPoint (int axis, CoalCoordType coord)
{
    if (axis < 0 || axis >= COAL_AXIS_COUNT)
	return;

    m_point[axis] = coord;
}

void CoalCoord::GetCoord (CoalCoordType& rx, CoalCoordType& ry, CoalCoordType& rz)
{
    rx = m_point[0];
    ry = m_point[1];
    rz = m_point[2];
}

void CoalCoord::SetCoord (CoalCoordType x, CoalCoordType y, CoalCoordType z)
{
    m_point[0] = x;
    m_point[1] = y;
    m_point[2] = z;
}

// Calculate the distance from this point to otherCoord
CoalCoordType CoalCoord::GetDistanceTo (const CoalCoord &otherCoord)
{
    CoalCoordType dx = fabs (otherCoord.GetX () - GetX ());
    CoalCoordType dy = fabs (otherCoord.GetY () - GetY ());

    return sqrt (dx * dx + dy * dy);
}

