// -*-C++-*-
// coord.h (imported from forge/servers/pangea/MapLaw/MapCoord.h)
//
//  The WorldForge Project
//  Copyright (C) 2000  The WorldForge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about WorldForge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//

// DESCRIPTION:
// A single coordinate
//


#ifndef __COORD_H
#define __COORD_H

#include <stl.h>
#include <string>

using namespace std;

typedef double CoalCoordType;

// How many dimensions we use
const int COAL_AXIS_COUNT = 3;

// Lots of happy little 'const' declarations to get rid of
// all those warning messages...

class CoalCoord
{
public:
    CoalCoord();
    CoalCoord(CoalCoordType x, CoalCoordType y)
	{ SetCoord(x, y, 0.0); }
    CoalCoord(CoalCoordType x, CoalCoordType y, CoalCoordType z)
	{ SetCoord(x, y, z); }
    virtual ~CoalCoord();

public:
    CoalCoordType GetPoint (int axis) const;
    void SetPoint (int axis, CoalCoordType coord);
    void GetCoord (CoalCoordType& rx, CoalCoordType& ry, CoalCoordType& rz);
    void SetCoord (CoalCoordType x, CoalCoordType y, CoalCoordType z);
    void SetCoord (CoalCoordType x, CoalCoordType y)
        { SetCoord (x, y, 0.0); }

    CoalCoordType GetX () const { return m_point[0]; }
    CoalCoordType GetY () const { return m_point[1]; }
    CoalCoordType GetZ () const { return m_point[2]; }

    void SetX (CoalCoordType x) { m_point[0] = x; }
    void SetY (CoalCoordType y) { m_point[1] = y; }
    void SetZ (CoalCoordType z) { m_point[2] = z; }

    // Are we inside or outside the coordinate system?
    bool IsValid() const { return (m_valid == true); }
    void SetValid(bool bValid = true) { m_valid = (bValid != false); }
    void SetInvalid() { m_valid = false; }

    void Add(CoalCoord coord);
    void Subtract(CoalCoord coord);

    CoalCoordType GetDistanceTo (const CoalCoord &otherCoord);

    // Operators
    const CoalCoord operator+ (const CoalCoord &coord2);
    const CoalCoord operator- (const CoalCoord &coord2);

    CoalCoord& operator= (const CoalCoord &copy);
    CoalCoord& operator+= (const CoalCoord &coord2);
    CoalCoord& operator-= (const CoalCoord &coord2);

    bool operator== (CoalCoord& coord2);
    bool operator== (const CoalCoord& coord2) const;
	bool operator!= (CoalCoord& coord2);
    bool operator!= (const CoalCoord& coord2) const;
    bool operator> (CoalCoord& coord2);
    bool operator>= (CoalCoord& coord2);
    bool operator< (CoalCoord& coord2);
    bool operator<= (CoalCoord& coord2);

private:
    CoalCoordType m_point[COAL_AXIS_COUNT];
    bool m_valid;

};

typedef vector<CoalCoord> CoalCoordArray;

#endif  // __COORD_H
