// -*-C++-*-
// isoloader.cpp
//
//  The WorldForge Project
//  Copyright (C) 2000  The WorldForge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about WorldForge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//

#include <fstream>
#include <iostream.h>
#include "isoloader.h"

using namespace std;

// a little helper ...
void skipEOL(ifstream& fin)
{
    int mark = fin.tellg();
    char nl;
    fin.get(nl);
    while(nl=='\n'){
        mark = fin.tellg();
        fin.get(nl);
    }
    fin.seekg(mark);
}   

// Test for tile graphics, based on the filename.
bool ObjectIsTile (string filename)
{
    if (filename.empty())
        return false;

    // Check for tiles with same rules as UClient
    if((filename.find("ground") != string::npos) ||
       (filename.find("floor") != string::npos) ||
       (filename.find("tiles") != string::npos) ||
       (filename.find("road") != string::npos) ||
       (filename.find("liquid") != string::npos))
        return true;

    return false;
}

// Load an isoclient map
bool CoalIsoLoader::LoadMap (const char *mapfile)
{
    int num_tiles;
    int mark;

    ifstream fin;
    fin.open(mapfile);
    if(!fin)
        return false;

    char* head = new char[15];
    if(!fin.read(head,sizeof(char)*5)) { delete []head; return 0;}

    // Map: 32 32 70 36
    if(!strstr(head,"Map: "))
    {
        cerr << "no valid map file...." << endl;
        return false;
    }

    int map_width, map_height;
    int xs,ys;

    // Get map size
    fin >> map_width >> map_height >> xs >> ys;

    // Get name of the map
    char n[256];
    fin.getline(n,255);

    // Get # of graphics files (TileList)
    do 
    {
        skipEOL(fin);
        mark = fin.tellg();
        if(!fin.read(head,sizeof(char)*10)) { delete []head; return false;}

        if(strstr(head,"TileList: "))
            fin >> num_tiles;
    } while (!strstr(head,"TileGfx: "));
      
    fin.seekg(mark);

    int d1, d2, d3;
    char space;
    char texturename[256];
    string typname;
    string texturefile;

    // Load graphic filenames
    CoalPolyGraphic *graphic;
    CoalFill *fill;
    for(int i = 0; i < num_tiles; i++)
    {
        fin.get(head, sizeof(char)*15,' ');
        fin >> d1 >> d2 >> d3;
        fin >> space;

        //rest of line is texturename
        fin.getline(texturename, 255);

        // Create a new graphic
        //cout << "<<< " << texturename << ">>>" << endl;
        graphic = new CoalPolyGraphic;
        graphic->filename = string (texturename);
        m_db->AddGraphic (graphic);

        // Also need a new fill
        fill = new CoalFill;
        fill->graphic = graphic;
        m_db->AddFill (fill);

        skipEOL(fin);
    }

    int id;
    int count;
    int num = 0;
    string idStr;
    CoalRegion *region;

    cout << "Parsing map regions" << endl;
    for(int y = 0; y < map_height; y++)
    {
        num++;

        for(int x = 0; x < map_width; x++)
        {
            fin.get(head, sizeof(char)*15,' ');
            fin >> count;

            for(int n = 0; n < count; n++)
            {
                fin >> id >> d1 >> d2 >> d3;
                graphic = (CoalPolyGraphic*)m_db->GetGraphic (id);
                if (!graphic)
                    continue;

                // Map iso tile coords to coal tile coords
                CoalCoord pos (y, map_width - x, 0);

                if (ObjectIsTile (graphic->filename))
                {
                    // Kinda painful and wasteful, but create a new
                    // CoalRegion for every single tile.
                    region = new CoalRegion;
                    fill = m_db->GetFill (id);
                    region->SetFill (fill);
                    region->MakeRect (pos, 2, 2);
                    m_db->AddRegion (region);
                }
                else
                {
                    CoalObject *object = new CoalObject;
                    object->anchor = pos;
                    object->graphic = graphic;
                    m_db->AddObject (object);
                }
            }
        }
    }

    cout << "Total region count: " << m_db->GetRegionCount () << endl;
  
    fin.close();

    return true;
}

bool CoalIsoLoader::LoadMap (const char *mapbuffer, int buffersize)
{
    return false;
}

bool CoalIsoLoader::SaveMap (const char *mapfile, int save_flags,
                          CoalShape *partial_map)
{
    return true;
}
