// -*-C++-*-
// maploader.h
//
//  The WorldForge Project
//  Copyright (C) 2000  The WorldForge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about WorldForge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//

// Abstract base class for loading different map formats and
// querying the contents

#ifndef __MAPLOADER_H
#define __MAPLOADER_H

#include "coordsystem.h"
#include "database.h"

// List of map loaders supported.  If you derive a new class from
// CoalMapLoader, don't forget to add an entry here...
typedef enum
{
    COAL_MAP_ISO,
    COAL_MAP_MIM,
    COAL_MAP_BLADE,
} CoalMapType;

struct CoalMapHint
{
    CoalMapType mapType;
    int mapWidth;
    int mapHeight;
};

class CoalMapLoader
{
public:
    // Doesn't make sense to create a loader without associating
    // it to a database, so force it right here in the constructor.
    // Use a & reference to make sure it isn't NULL, so we don't have
    // to check for NULL everytime we use it.
    CoalMapLoader (CoalDatabase &db)
        { SetDatabase (&db); SetMediaFilter (COAL_MEDIA_ALL); }
    virtual ~CoalMapLoader () {}

    virtual bool LoadMap (const char *mapfile) = 0;
    virtual bool LoadMap (const char *mapbuffer, int buffersize) = 0;
    virtual bool SaveMap (const char *mapfile, int save_flags = 0,
                          CoalShape *partial_map = NULL) = 0;
    virtual bool CleanupLoader () = 0;

    // Only set a new database if non-NULL.  Must have a valid
    // database at all times.  Only need to use SetDatabase () if
    // you need to switch databases in the middle of things (rare!)
    // Should probably remove SetDatabase() because it implies that
    // it's okay to use it.  (c:
    virtual void SetDatabase (CoalDatabase *db) { if (db) m_db = db; }
    virtual CoalDatabase* GetDatabase () { return m_db; }

    void SetMediaFilter (CoalMediaType filter)
        { m_media_filter = filter; }

protected:
    CoalDatabase *m_db;
    CoalMediaType m_media_filter;
};

#endif  // __MAPLOADER_H
